/*
 * The class contains a function runGame() to initialize a new game.
 * Starting a new game enables the following sequence:
 * -Get the height and width of the playing field
 * -Create an instance of class PlayingField
 * -Create two player instances of class Player
 * -Assign turn to the first player
 * -Start a new thread to run the game itself.
 *
 * The game sequence:
 * -while(true)
 *     -Wait for input confirmation from PlayingField of Player 1
 *     -Drop the Disc
 *     -Check for winning move : winning move == true --> break loop
 *     -Check for PlayingField full : full == true --> break loop
 *     -Assign turn to Player 2 and enable/disable controls
 *     -Wait for input confirmation from PlayingField of Player 2
 *     -Drop the Disc
 *     -Check for winning move : winning move == true --> break loop
 *     -Check for PlayingField full : full == true --> break loop
 *     -Assign turn to Player 1 and enable/disable controls
 *
 * -Create GameResult instance
 * -Call addGame(GameResult)
 * -End game animation
 */
package GUI;
//import Game.AITestIvo;
import ScoreManagement.GameResult;
import ScoreManagement.ScoreManagement;
import ScoreManagement.User;
import Game.ComputerPlayer;
import Game.Disc;
import Game.HumanPlayer;
import Game.Player;
import Game.PlayingField;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.KeyEvent;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

/**
 * Handles the general game logic and game user interface
 */
public class Game extends javax.swing.JDialog implements Runnable {

    private Player player1; //Variable to store instance player 1
    private Player player2; //Variable to store instance player 2
    public PlayingField field;
    Thread runThread;
    JLabel[][] cells; //Graphic representation of cells/discs, indexed by x,y
    public static Boolean noanimation = false; //Disable animations (for testing)

    /**
     * Create and start a new game and window
     * @param parent The parent window for this dialog window
     * @param modal Whether to show this window as a modal window
     */
    public Game(java.awt.Frame parent, boolean modal) {
	super(parent, modal);
	initComponents();
	jButton2.setVisible(false);
	runGame();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        panelContent = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jPanelField = new javax.swing.JPanel();
        jPanel4 = new javax.swing.JPanel();
        jlNewDisc = new javax.swing.JLabel();
        jPanel5 = new javax.swing.JPanel();
        panelPlayer1 = new javax.swing.JPanel();
        jLabelPlayer1 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jPanel6 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        panelPlayer2 = new javax.swing.JPanel();
        jLabelPlayer2 = new javax.swing.JLabel();
        jPanel3 = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        jPanel7 = new javax.swing.JPanel();
        jButton2 = new javax.swing.JButton();
        labelTitle = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                formKeyPressed(evt);
            }
        });

        mainPanel.setBackground(new java.awt.Color(52, 127, 191));
        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setLayout(new java.awt.BorderLayout());

        panelContent.setName("panelContent"); // NOI18N
        panelContent.setLayout(new java.awt.BorderLayout());

        jPanel1.setBackground(new java.awt.Color(255, 255, 255));
        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setPreferredSize(new java.awt.Dimension(200, 564));
        jPanel1.setLayout(new java.awt.BorderLayout());

        jPanelField.setName("jPanelField"); // NOI18N
        jPanelField.setPreferredSize(new java.awt.Dimension(500, 413));
        jPanelField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                jPanelFieldMousePressed(evt);
            }
        });
        jPanelField.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                jPanelFieldMouseMoved(evt);
            }
        });

        javax.swing.GroupLayout jPanelFieldLayout = new javax.swing.GroupLayout(jPanelField);
        jPanelField.setLayout(jPanelFieldLayout);
        jPanelFieldLayout.setHorizontalGroup(
            jPanelFieldLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 528, Short.MAX_VALUE)
        );
        jPanelFieldLayout.setVerticalGroup(
            jPanelFieldLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 504, Short.MAX_VALUE)
        );

        jPanel1.add(jPanelField, java.awt.BorderLayout.CENTER);

        jPanel4.setBackground(java.awt.SystemColor.controlLtHighlight);
        jPanel4.setName("jPanel4"); // NOI18N
        jPanel4.setPreferredSize(new java.awt.Dimension(528, 60));
        jPanel4.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                jPanel4MousePressed(evt);
            }
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jPanel4MouseClicked(evt);
            }
        });
        jPanel4.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                jPanel4MouseMoved(evt);
            }
        });
        jPanel4.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                jPanel4KeyPressed(evt);
            }
        });
        jPanel4.setLayout(null);

        jlNewDisc.setBackground(java.awt.Color.yellow);
        jlNewDisc.setIcon(new javax.swing.ImageIcon(getClass().getResource("/GUI/resources/gap_white.png"))); // NOI18N
        jlNewDisc.setName("jlNewDisc"); // NOI18N
        jlNewDisc.setOpaque(true);
        jPanel4.add(jlNewDisc);
        jlNewDisc.setBounds(107, 0, 60, 60);

        jPanel1.add(jPanel4, java.awt.BorderLayout.NORTH);

        panelContent.add(jPanel1, java.awt.BorderLayout.CENTER);

        jPanel5.setName("jPanel5"); // NOI18N
        jPanel5.setPreferredSize(new java.awt.Dimension(150, 261));
        jPanel5.setLayout(new java.awt.BorderLayout());

        panelPlayer1.setBackground(new java.awt.Color(2, 139, 16));
        panelPlayer1.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        panelPlayer1.setName("panelPlayer1"); // NOI18N

        jLabelPlayer1.setFont(new java.awt.Font("Tahoma", 1, 14));
        jLabelPlayer1.setText("Ivo");
        jLabelPlayer1.setName("jLabelPlayer1"); // NOI18N

        jPanel2.setBackground(new java.awt.Color(220, 220, 220));
        jPanel2.setName("jPanel2"); // NOI18N

        jLabel2.setText("Player 1");
        jLabel2.setName("jLabel2"); // NOI18N

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addContainerGap(99, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout panelPlayer1Layout = new javax.swing.GroupLayout(panelPlayer1);
        panelPlayer1.setLayout(panelPlayer1Layout);
        panelPlayer1Layout.setHorizontalGroup(
            panelPlayer1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addGroup(panelPlayer1Layout.createSequentialGroup()
                .addGap(10, 10, 10)
                .addComponent(jLabelPlayer1, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        panelPlayer1Layout.setVerticalGroup(
            panelPlayer1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelPlayer1Layout.createSequentialGroup()
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabelPlayer1)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel5.add(panelPlayer1, java.awt.BorderLayout.SOUTH);

        panelContent.add(jPanel5, java.awt.BorderLayout.WEST);

        jPanel6.setName("jPanel6"); // NOI18N
        jPanel6.setPreferredSize(new java.awt.Dimension(150, 237));
        jPanel6.setLayout(new java.awt.BorderLayout());

        jButton1.setText("End Game");
        jButton1.setFocusable(false);
        jButton1.setName("jButton1"); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel6.add(jButton1, java.awt.BorderLayout.NORTH);

        panelPlayer2.setBackground(new java.awt.Color(255, 255, 255));
        panelPlayer2.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        panelPlayer2.setName("panelPlayer2"); // NOI18N

        jLabelPlayer2.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        jLabelPlayer2.setText("Computer");
        jLabelPlayer2.setName("jLabelPlayer2"); // NOI18N

        jPanel3.setBackground(new java.awt.Color(220, 220, 220));
        jPanel3.setName("jPanel3"); // NOI18N

        jLabel5.setText("Player 2");
        jLabel5.setName("jLabel5"); // NOI18N

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel5)
                .addContainerGap(99, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel5)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout panelPlayer2Layout = new javax.swing.GroupLayout(panelPlayer2);
        panelPlayer2.setLayout(panelPlayer2Layout);
        panelPlayer2Layout.setHorizontalGroup(
            panelPlayer2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addGroup(panelPlayer2Layout.createSequentialGroup()
                .addGap(10, 10, 10)
                .addComponent(jLabelPlayer2, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        panelPlayer2Layout.setVerticalGroup(
            panelPlayer2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelPlayer2Layout.createSequentialGroup()
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabelPlayer2)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel6.add(panelPlayer2, java.awt.BorderLayout.SOUTH);

        jPanel7.setName("jPanel7"); // NOI18N
        jPanel7.setLayout(new java.awt.BorderLayout());

        jButton2.setText("Play Again");
        jButton2.setName("jButton2"); // NOI18N
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel7.add(jButton2, java.awt.BorderLayout.PAGE_START);

        jPanel6.add(jPanel7, java.awt.BorderLayout.CENTER);

        panelContent.add(jPanel6, java.awt.BorderLayout.EAST);

        mainPanel.add(panelContent, java.awt.BorderLayout.CENTER);

        labelTitle.setFont(new java.awt.Font("Tahoma", 1, 24));
        labelTitle.setForeground(new java.awt.Color(255, 255, 255));
        labelTitle.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        labelTitle.setText("Play Game");
        labelTitle.setName("labelTitle"); // NOI18N
        mainPanel.add(labelTitle, java.awt.BorderLayout.PAGE_START);

        getContentPane().add(mainPanel, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /* End game button handler */
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
	dispose();
    }//GEN-LAST:event_jButton1ActionPerformed

    /* Keyboard input handler, handles keyboard game controls */
    private void formKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_formKeyPressed
	if (!Options.getControlKeyboard()) {
	    return;
	}
	int kc = evt.getKeyCode();
	Player turn = field.getTurn();
	if (!(turn instanceof HumanPlayer)) {
	    return;
	}
	HumanPlayer hp = (HumanPlayer) turn;
	int w = field.getWidth();

	if (turn == player1) {
	    if (kc == KeyEvent.VK_DOWN || kc == KeyEvent.VK_ENTER || kc == KeyEvent.VK_SPACE) {
		if (hp.selectedColumn >= 0 && hp.selectedColumn < w) {
		    hp.putDisc(hp.selectedColumn);
		}
	    } else if (kc == KeyEvent.VK_LEFT) {
		hp.selectedColumn = (hp.selectedColumn <= 0) ? 0 : hp.selectedColumn - 1;
		jlNewDisc.setLocation(hp.selectedColumn * jlNewDisc.getWidth(), 0);
	    } else if (kc == KeyEvent.VK_RIGHT) {
		hp.selectedColumn = (hp.selectedColumn + 1 >= w) ? w - 1 : hp.selectedColumn + 1;
		jlNewDisc.setLocation(hp.selectedColumn * jlNewDisc.getWidth(), 0);
	    }
	} else {
	    if (kc == KeyEvent.VK_S) {
		if (hp.selectedColumn >= 0 && hp.selectedColumn < w) {
		    hp.putDisc(hp.selectedColumn);
		}
	    } else if (kc == KeyEvent.VK_A) {
		hp.selectedColumn = (hp.selectedColumn <= 0) ? 0 : hp.selectedColumn - 1;
		jlNewDisc.setLocation(hp.selectedColumn * jlNewDisc.getWidth(), 0);
	    } else if (kc == KeyEvent.VK_D) {
		hp.selectedColumn = (hp.selectedColumn + 1 >= w) ? w - 1 : hp.selectedColumn + 1;
		jlNewDisc.setLocation(hp.selectedColumn * jlNewDisc.getWidth(), 0);
	    }
	}
    }//GEN-LAST:event_formKeyPressed
    /* Keyboard input handler, handles keyboard game controls by forwarding to formKeyPressed */
    private void jPanel4KeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jPanel4KeyPressed
	formKeyPressed(evt);
    }//GEN-LAST:event_jPanel4KeyPressed
    /* Top-panel mouse click handler to set focus to accept keyboard input */
    private void jPanel4MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jPanel4MouseClicked
	jPanel4.requestFocus();
    }//GEN-LAST:event_jPanel4MouseClicked

    /* Game panel mouse move handler for mouse game controls */
    private void jPanelFieldMouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jPanelFieldMouseMoved
	if (Options.getControlKeyboard()) {
	    return;
	}
	Player turn = field.getTurn();
	if (!(turn instanceof HumanPlayer)) {
	    return;
	}
	HumanPlayer hp = (HumanPlayer) turn;
	int w = field.getWidth();
	int x = evt.getX() / cells[0][0].getWidth();
	if (x < 0) {
	    x = 0;
	}
	if (x >= w) {
	    x = w - 1;
	}
	hp.selectedColumn = x;
	jlNewDisc.setLocation(hp.selectedColumn * jlNewDisc.getWidth(), 0);
    }//GEN-LAST:event_jPanelFieldMouseMoved
    /* Game panel mouse click handler for mouse game controls (drop disk) */
    private void jPanelFieldMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jPanelFieldMousePressed
	if (Options.getControlKeyboard()) {
	    return;
	}
	Player turn = field.getTurn();
	if (!(turn instanceof HumanPlayer)) {
	    return;
	}
	HumanPlayer hp = (HumanPlayer) turn;
	int w = field.getWidth();
	if (hp.selectedColumn < 0 && hp.selectedColumn >= w) {
	    return;
	}
	hp.putDisc(hp.selectedColumn);
    }//GEN-LAST:event_jPanelFieldMousePressed
    /* Redirect top-panel mouse events to game panel event handler */
    private void jPanel4MouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jPanel4MouseMoved
	jPanelFieldMouseMoved(evt);
    }//GEN-LAST:event_jPanel4MouseMoved
    /* Redirect top-panel mouse events to game panel event handler */
    private void jPanel4MousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jPanel4MousePressed
	jPanelFieldMousePressed(evt);
    }//GEN-LAST:event_jPanel4MousePressed

    /* Play again button handler */
private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
    dispose();
    (new Game(Application.getApplication().getMainFrame(), true)).setVisible(true);
}//GEN-LAST:event_jButton2ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabelPlayer1;
    private javax.swing.JLabel jLabelPlayer2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanelField;
    private javax.swing.JLabel jlNewDisc;
    private javax.swing.JLabel labelTitle;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JPanel panelContent;
    private javax.swing.JPanel panelPlayer1;
    private javax.swing.JPanel panelPlayer2;
    // End of variables declaration//GEN-END:variables

    /* Destroy this class instance (abort game thread and destroy form) */
    @Override
    public void dispose() {
	quitGame();
	super.dispose();
    }

    //Getters
    /**
     * Initializes the graphic components, player instances and starts the game logic
     * Pre: true
     * Post: a game has been initialized and started
     */
    public void runGame() {
	int width = Options.getFieldWidth();
	int height = Options.getFieldHeight();

	ImageIcon icon = new ImageIcon(getClass().getResource("/GUI/resources/gap.png"));
	Dimension size = new Dimension(icon.getIconWidth() * width, icon.getIconHeight() * height);
	jPanelField.setPreferredSize(size);
	jPanelField.setMaximumSize(size);
	jPanelField.setMinimumSize(size);
	jPanelField.setSize(size);
	jPanelField.setLayout(null);
	cells = new JLabel[width][height];
	for (int x = 0; x < width; x++) {
	    for (int y = 0; y < height; y++) {
		JLabel label = new JLabel();
		label.setIcon(icon);
		label.setHorizontalAlignment(SwingConstants.CENTER);
		label.setVerticalAlignment(SwingConstants.CENTER);
		label.setBounds(x * icon.getIconWidth(), (height - y - 1) * icon.getIconHeight(), icon.getIconWidth(), icon.getIconHeight());
		label.setOpaque(true);
		jPanelField.add(label);
		cells[x][y] = label;
	    }
	}
	setSize(new Dimension(size.width + 315, size.height + 155));

	User user1 = Options.getPlayer1();
	User user2 = Options.getPlayer2();

	player1 = getPlayerForUser(user1, "yellow");
	player2 = getPlayerForUser(user2, "red");

	field = new PlayingField(width, height, player1, player2, player1);

	jLabelPlayer1.setText(player1.user.getName());
	jLabelPlayer2.setText(player2.user.getName());

	runThread = new Thread(this);
	runThread.start();
    }

    /* Create a Player instance for the given User playing the given Color */
    private Player getPlayerForUser(User u, String color) {
	if (u.isComputer()) {
	    return new ComputerPlayer(this, u, color);
	} else {
	    //return new AITestIvo(this, u, color);
	    return new HumanPlayer(this, u, color);
	}
    }

    /* Abort the game logic thread */
    private void quitGame() {
	runThread.stop();
    }

    /*
     * Main game logic procedure ran by the runThread from runGame()
     */
    @Override
    public void run() {
	try {
	    processGame();
	} catch (InterruptedException ex) {
	}
    }

    /*
     * General game logic
     * Repeat:
     *	Get current player
     *  Update current player indicator
     *  Enable disk position indicator
     *  Ask the player to make a move
     *  Drop disk animation
     *  Check if someone has won the game or the field is full
     *   Store the result in the database
     *   Display appropriate animation
     *   Break out of loop
     */
    private void processGame() throws InterruptedException {
	while (true) {
	    Player turn = field.getTurn();
	    System.out.println("Turn: player" + (turn == player1 ? "1" : "2") + " is " + turn.user.getName());
	    //Show the current player
	    panelPlayer1.setBackground(turn == player1 ? Color.green : Color.white);
	    panelPlayer2.setBackground(turn == player2 ? Color.green : Color.white);
	    //Ask player to make a move
	    if (!noanimation && turn instanceof HumanPlayer) {
		//Determine the player color
		Color pc = "red".equals(turn.color) ? Color.red : Color.yellow;
		//Show the disk above the column for some time
		jlNewDisc.setBackground(pc);
		jlNewDisc.setVisible(true);
		jlNewDisc.setLocation(((HumanPlayer) turn).selectedColumn * jlNewDisc.getWidth(), 0);
		requestFocus();
	    }
	    long tstart = Calendar.getInstance().getTime().getTime();
	    turn.doMove();
	    long tdiff = Calendar.getInstance().getTime().getTime() - tstart;
	    System.out.println("Move took " + Long.toString(tdiff) + " ms");
	    //Debug: print the actual status of the field (which should be the same as the GUI)
	    //field.printField();
	    //Debug check: did the player make a move?
	    if (field.getLastMove() == null || field.getLastMove().getOwner() != turn) {
		System.out.println("No move from player " + turn.user.getName());
		continue;
	    }
	    //Animate the disk falling into position
	    dropDiscAnimation(field.getLastMove());
	    //Check if the game has ended (winner or draw)
	    Player winning = field.checkWin();
	    if (winning != null || field.isFull()) {
		GameResult result = new GameResult(player1.user, player2.user,
			winning == null ? null : winning.user, getDate(),
			field.countDiscs());
		ScoreManagement.addGame(result);
		endGameAnimation(winning);
		break;
	    }
	}
    }

    /**
     * Display animation of disk falling into position (and leave it there)
     * Pre: Disc.position != null && Disc.Player != null
     * Post: The animation has been played
     */
    private void dropDiscAnimation(Disc disc) throws InterruptedException {
	System.out.println("Disc is at " + Integer.toString(disc.getColumn()) + "x" + Integer.toString(disc.getRow()) + " owned by " + disc.getOwner().user.getName());
	//Determine the player color
	Color pc = "red".equals(disc.getOwner().color) ? Color.red : Color.yellow;
	if (noanimation) {
	    cells[disc.getColumn()][disc.getRow()].setBackground(pc);
	    return;
	}
	//Show the disk above the column for some time
	jlNewDisc.setLocation(disc.getColumn() * jlNewDisc.getWidth(), 0);
	jlNewDisc.setBackground(pc);
	jlNewDisc.setVisible(true);
	Thread.sleep(200);
	jlNewDisc.setVisible(false);
	//Show the disk dropping in position (and leave it there)
	int h = field.getheight();
	Color bc = cells[disc.getColumn()][h - 1].getBackground();
	if (!cells[disc.getColumn()][disc.getRow()].getBackground().equals(bc)) {
	    System.out.println("Cell not empty");
	}
	for (int y = h - 1; y >= disc.getRow(); y--) {
	    if (y + 1 < h) {
		cells[disc.getColumn()][y + 1].setBackground(bc);
	    }
	    cells[disc.getColumn()][y].setBackground(pc);
	    Thread.sleep(100);
	}
	disc.playSound();
    }

    /**
     * Display game result to the user:
     *	If no one wins, display a pop-up message
     *  If someone wins, blink user name and winning sequence of discs
     * Pre: field.checkWin() != null || isFull()
     * Post: The animation has been played
     */
    private void endGameAnimation(Player player) throws InterruptedException {
	if (player == null) {
	    JOptionPane.showMessageDialog(null, "No one wins");
	    jButton2.setVisible(true);
	    System.out.println("No one wins");
	} else {
	    System.out.println(player.user.getName() + " wins");
	    if (player != null && !player.user.isComputer()) {
		playSound();
	    }
	    jButton2.setVisible(true);
	    JPanel playerpanel = (player == player1) ? panelPlayer1 : panelPlayer2;
	    Disc[] sequence = field.getWinningSequence();
	    Color c = cells[sequence[0].getColumn()][sequence[0].getRow()].getBackground();
	    while (true) {
		playerpanel.setBackground(Color.green);
		for (Disc d : sequence) {
		    cells[d.getColumn()][d.getRow()].setBackground(c);
		}
		Thread.sleep(500);
		playerpanel.setBackground(Color.gray);
		for (Disc d : sequence) {
		    cells[d.getColumn()][d.getRow()].setBackground(Color.white);
		}
		Thread.sleep(500);
	    }
	}

    }

    /* Get a string representing the current date */
    private String getDate() {
	Calendar date = Calendar.getInstance();
	SimpleDateFormat dateformatter = new SimpleDateFormat("dd-MM-yyyy");
	return dateformatter.format(date.getTime());
    }

    /**
     * Play a sound when someone has won
     * Pre: true
     * Post: A sound has been played
     */
    private void playSound() {
	try {
	    URL ais = this.getClass().getResource("/GUI/resources/won.wav");
	    AudioInputStream audioIn = AudioSystem.getAudioInputStream(ais);
	    // Get a sound clip resource.
	    Clip clip = AudioSystem.getClip();
	    // Open audio clip and load samples from the audio input stream.
	    clip.open(audioIn);
	    clip.start();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
